/*
 * ContainerRegistry.cpp
 *
 *  Created on: 19. 8. 2017
 *	  Author: Jan Travnicek
 */

#include <registry/ContainerRegistry.hpp>
#include <algorithm>
#include <typeinfo>

namespace abstraction {

bool ContainerRegistry::hasAbstraction ( const std::string & container ) {
	return getEntries ( ).count ( container );
}

std::shared_ptr < abstraction::OperationAbstraction > ContainerRegistry::getAbstraction ( const std::string & container, const std::string & param ) {
	auto group = getEntries ( ).find ( container );
	if ( group == getEntries ( ).end ( ) )
		throw std::invalid_argument ( "Entry " + container + " not available" );

	for ( const ext::pair < std::string, std::shared_ptr < Entry > > & entry : group->second )
		if ( ext::is_same_type ( param, ext::erase_template_info ( entry.first ) ) )
			return entry.second->getAbstraction ( );

	throw std::invalid_argument ( "Entry for " + container + " parametrized with " + param + " not available." );
}

ext::set < std::string > ContainerRegistry::listOverloads ( const std::string & container ) {
	auto group = getEntries ( ).find ( container );
	if ( group == getEntries ( ).end ( ) )
		throw std::invalid_argument ( "Entry " + container + " not available" );

	ext::set < std::string > res;
	for ( const ext::pair < std::string, std::shared_ptr < Entry > > & entry : group->second )
		res.insert ( entry.first );

	return res;
}

ext::set < std::string > ContainerRegistry::list ( ) {
	ext::set < std::string > res;

	for ( const std::pair < const std::string, ext::vector < ext::pair < std::string, std::shared_ptr < Entry > > > > & groups : getEntries ( ) )
		res.insert ( groups.first );

	return res;
}

} /* namespace abstraction */
