/*
 * NonterminalUnitRuleCycle.h
 *
 *  Created on: 22. 3. 2014
 *	  Author: Tomas Pecka
 */

#ifndef NONTERMINAL_UNIT_RULE_CYCLE_H_
#define NONTERMINAL_UNIT_RULE_CYCLE_H_

#include <set>
#include <deque>

#include <exception/CommonException.h>

namespace grammar {

namespace properties {

/**
 * Implements algorithms from Melichar, chapter 3.3
 */
class NonterminalUnitRuleCycle {
public:
	/**
	 * Retrieves set N = {B : A->^* B} for given grammar and nonterminal
	 *
	 * Source: Melichar, algorithm 2.6, step 1
	 *
	 * @param grammar grammar
	 * @param nonterminal nonterminal
	 * @return set of nonterminals for which we can be derived from giveUnitRuleCyclen nonterminals in finite number of steps
	 */
	template<class T, class SymbolType>
	static ext::set<SymbolType> getNonterminalUnitRuleCycle(const T& grammar, const SymbolType& nonterminal);
};

template<class T, class SymbolType >
ext::set<SymbolType> NonterminalUnitRuleCycle::getNonterminalUnitRuleCycle(const T& grammar, const SymbolType& nonterminal) {
	if(grammar.getNonterminalAlphabet().count(nonterminal) == 0) {
		throw exception::CommonException("Nonterminal symbol \"" + ext::to_string ( nonterminal ) + "\" is not present in grammar.");
	}

	ext::deque<ext::set<SymbolType>> Ni;
	Ni.push_back(ext::set<SymbolType>{nonterminal});
	int i = 1;

	while(true) {
		Ni.push_back(Ni.at(i-1));
		for(const auto &rule : grammar.getRawRules()) {
			const SymbolType& lhs = rule.first;

			for(const auto& rhs : rule.second) {
				if(Ni.at(i-1).count(lhs) && rhs.size() == 1 && grammar.getNonterminalAlphabet().count(rhs.front())) {
					Ni.at(i).insert(rhs.front());
				}
			}
		}

		if(Ni.at(i) == Ni.at(i-1))
			break;
		
		i += 1;
	}

	return Ni.at(i);
}

} /* namespace properties */

} /* namespace grammar */

#endif /* NONTERMINAL_UNIT_RULE_CYCLE_H_ */
