/*
 * GrammarPropertiesCFG.h
 *
 *  Created on: 22. 3. 2014
 *	  Author: Tomas Pecka
 */

#ifndef PRODUCTIVE_NONTERMINALS_H_
#define PRODUCTIVE_NONTERMINALS_H_

#include <set>
#include <deque>
#include <algorithm>

#include <grammar/Grammar.h>

namespace grammar {

namespace properties {

/**
 * Implements algorithms from Melichar, chapter 3.3
 */
class ProductiveNonterminals {
public:
	/**
	 * Implements steps 1 through 3 in Melichar 3.6
	 */
	template<class T, class SymbolType = typename grammar::SymbolTypeOfGrammar < T > >
	static ext::set<SymbolType> getProductiveNonterminals( const T & grammar );
};

template<class T, class SymbolType >
ext::set<SymbolType> ProductiveNonterminals::getProductiveNonterminals( const T & grammar ) {
	// 1.
	ext::deque<ext::set<SymbolType>> Ni;
	Ni.push_back( ext::set<SymbolType>( ) );

	int i = 1;

	// 2.
	while( true ) {
		Ni.push_back( Ni.at( i - 1 ) );

		for( const auto & rule : grammar.getRawRules( ) ) {
			for( const auto & rhs : rule.second ) {
				if( std::all_of( rhs.begin( ), rhs.end( ), [ i, Ni, grammar ]( const SymbolType & symbol ) -> bool {
					return Ni.at( i - 1 ) . count( symbol ) || grammar.getTerminalAlphabet( ). count( symbol );
				} ) )
					Ni.at( i ).insert( rule.first );
			}
		}

		if( Ni.at( i ) == Ni.at( i - 1 ) )
			break;

		i = i + 1;
	}

	// 3.
	return Ni.at( i );
}

} /* namespace properties */

} /* namespace grammar */

#endif /* PRODUCTIVE_NONTERMINALS_H_ */
