/*
 * RecursiveNonterminal.h
 *
 *  Created on: 16. 11. 2015
 *	  Author: Jan Travnicek
 */

#ifndef RECURSIVE_NONTERMINAL_H_
#define RECURSIVE_NONTERMINAL_H_

#include "NullableNonterminals.h"

#include <set>
#include <deque>

#include <exception/CommonException.h>

namespace grammar {

namespace properties {

/**
 * Implements algorithms from Melichar, chapter 3.3
 */
class RecursiveNonterminal {
public:
	/**
	 * Retrieves A \in { B : A->^+ B \alpha, where \alpha \in (NuT)* } for given grammar and nonterminal
	 *
	 * @param grammar grammar
	 * @param nonterminal nonterminal
	 * @return bool which denote whether the nonterminal is recursive in the grammar
	 */
	template < class T, class SymbolType  >
	static bool isNonterminalRecursive ( const T & grammar, const SymbolType & nonterminal );

};

template < class T, class SymbolType  >
bool RecursiveNonterminal::isNonterminalRecursive ( const T & grammar, const SymbolType & nonterminal ) {
	if ( grammar.getNonterminalAlphabet ( ).count ( nonterminal ) == 0 )
		throw exception::CommonException ( "Nonterminal symbol \"" + ( std::string ) nonterminal + "\" is not present in grammar." );

	ext::deque < ext::set < SymbolType > > Ni;
	Ni.push_back ( ext::set < SymbolType > { nonterminal } );
	unsigned i = 1;

	auto rawRules = grammar.getRawRules ( );
	auto nullable = grammar::properties::NullableNonterminals::getNullableNonterminals ( grammar );

	while ( i <= grammar.getNonterminalAlphabet ( ).size ( ) ) {
		Ni.push_back ( ext::set < SymbolType > { } );

		for ( const SymbolType & lhs : Ni.at ( i - 1 ) )
			if ( rawRules.find ( lhs ) != rawRules.end ( ) )
				for ( const ext::vector < SymbolType > & rhs : rawRules.find ( lhs )->second )
					for ( const SymbolType & rhsSymbol : rhs ) {
						if ( grammar.getTerminalAlphabet ( ).count ( rhsSymbol ) ) break;

						Ni.at ( i ).insert ( rhsSymbol );

						if ( !nullable.count ( rhsSymbol ) ) break;
					}



		if ( Ni.at ( i ).count ( nonterminal ) ) return true;

		i += 1;
	}

	return false;
}

} /* namespace properties */

} /* namespace grammar */

#endif /* RECURSIVE_NONTERMINAL_H_ */
