/*
 * UnreachableSymbols.h
 *
 *  Created on: 22. 3. 2014
 *	  Author: Tomas Pecka
 */

#ifndef UNREACHABLE_SYMBOLS_H_
#define UNREACHABLE_SYMBOLS_H_

#include <algorithm>
#include <deque>
#include <set>

#include <grammar/Grammar.h>

namespace grammar {

namespace properties {

/**
 * Implements algorithms from Melichar, chapter 3.3
 */
class UnreachableSymbols {
public:
	/**
	 * Implements
	 */
	template<class T, class SymbolType = typename grammar::SymbolTypeOfGrammar < T > >
	static ext::set<DefaultSymbolType> getUnreachableSymbols( const T & grammar );
};

template<class T, class SymbolType >
ext::set<DefaultSymbolType> UnreachableSymbols::getUnreachableSymbols( const T & grammar ) {
	// 1
	ext::deque<ext::set<DefaultSymbolType>> Vi;
	Vi.push_back( ext::set<DefaultSymbolType>( ) );
	Vi.at( 0 ).insert( grammar.getInitialSymbol( ) );

	int i = 1;

	// 2.
	while( true ) {
		Vi.push_back( Vi.at( i - 1 ) );

		for( const auto & rule : grammar.getRawRules( ) ) {
			if( Vi.at( i - 1 ).count( rule.first ) ) {
				for( const auto & rhs : rule.second ) {
					Vi.at( i ).insert( rhs.begin( ), rhs.end( ) );
				}
			}
		}


		if( Vi.at( i ) == Vi.at( i - 1 ) )
			break;

		i = i + 1;
	}

	// 3.
	return Vi.at( i );
}

} /* namespace properties */

} /* namespace grammar */

#endif /* UNREACHABLE_SYMBOLS_H_ */
