/*
 * GlushkovIndexate.h
 *
 *  Created on: 13. 3. 2014
 *	  Author: Tomas Pecka
 */

#ifndef GLUSHKOV_INDEXATE_H_
#define GLUSHKOV_INDEXATE_H_

#include <regexp/unbounded/UnboundedRegExp.h>
#include <regexp/RegExpFeatures.h>

#include <regexp/unbounded/UnboundedRegExpAlternation.h>
#include <regexp/unbounded/UnboundedRegExpConcatenation.h>
#include <regexp/unbounded/UnboundedRegExpElement.h>
#include <regexp/unbounded/UnboundedRegExpEmpty.h>
#include <regexp/unbounded/UnboundedRegExpEpsilon.h>
#include <regexp/unbounded/UnboundedRegExpIteration.h>
#include <regexp/unbounded/UnboundedRegExpSymbol.h>

namespace regexp {

/**
 * RegExp tree traversal utils for Glushkov algorithm.
 *
 */
class GlushkovIndexate {
public:
	/**
	 * @param re RegExp to index
	 * @return UnboundedRegExp with indexed elements
	 */
	template < class SymbolType >
	static regexp::UnboundedRegExp < ext::pair < SymbolType, int > > index ( const regexp::UnboundedRegExp < SymbolType > & re );

	template < class SymbolType >
	class Unbounded {
	public:
		static ext::rvalue_ref < regexp::UnboundedRegExpElement < ext::pair < SymbolType, int > > > visit(const regexp::UnboundedRegExpAlternation < SymbolType > & alternation, int & i);
		static ext::rvalue_ref < regexp::UnboundedRegExpElement < ext::pair < SymbolType, int > > > visit(const regexp::UnboundedRegExpConcatenation < SymbolType > & concatenation, int & i);
		static ext::rvalue_ref < regexp::UnboundedRegExpElement < ext::pair < SymbolType, int > > > visit(const regexp::UnboundedRegExpIteration < SymbolType > & iteration, int & i);
		static ext::rvalue_ref < regexp::UnboundedRegExpElement < ext::pair < SymbolType, int > > > visit(const regexp::UnboundedRegExpSymbol < SymbolType > & symbol, int & i);
		static ext::rvalue_ref < regexp::UnboundedRegExpElement < ext::pair < SymbolType, int > > > visit(const regexp::UnboundedRegExpEpsilon < SymbolType > & epsilon, int & i);
		static ext::rvalue_ref < regexp::UnboundedRegExpElement < ext::pair < SymbolType, int > > > visit(const regexp::UnboundedRegExpEmpty < SymbolType > & empty, int & i);
	};
};

template < class SymbolType >
UnboundedRegExp < ext::pair < SymbolType, int > > GlushkovIndexate::index ( const regexp::UnboundedRegExp < SymbolType > & re ) {
	int i = 1;

	return UnboundedRegExp < ext::pair < SymbolType, int > > ( regexp::UnboundedRegExpStructure < ext::pair < SymbolType, int > > ( re.getRegExp ( ).getStructure ( ).template accept < ext::rvalue_ref < regexp::UnboundedRegExpElement < ext::pair < SymbolType, int > > >, GlushkovIndexate::Unbounded < SymbolType > > ( i ) ) );
}

template < class SymbolType >
ext::rvalue_ref < regexp::UnboundedRegExpElement < ext::pair < SymbolType, int > > > GlushkovIndexate::Unbounded < SymbolType >::visit(const regexp::UnboundedRegExpAlternation < SymbolType > & alternation, int & i) {
	UnboundedRegExpAlternation < ext::pair < SymbolType, int > > * alt = new UnboundedRegExpAlternation < ext::pair < SymbolType, int > > ( );

	for ( const auto & element : alternation.getElements ( ) )
		alt->appendElement ( element->template accept < ext::rvalue_ref < regexp::UnboundedRegExpElement < ext::pair < SymbolType, int > > >, GlushkovIndexate::Unbounded < SymbolType > > ( i ) );

	return ext::rvalue_ref < UnboundedRegExpElement < ext::pair < SymbolType, int > > > ( alt );
}

template < class SymbolType >
ext::rvalue_ref < regexp::UnboundedRegExpElement < ext::pair < SymbolType, int > > > GlushkovIndexate::Unbounded < SymbolType >::visit(const regexp::UnboundedRegExpConcatenation < SymbolType > & concatenation, int & i) {
	UnboundedRegExpConcatenation < ext::pair < SymbolType, int > > * con = new UnboundedRegExpConcatenation < ext::pair < SymbolType, int > > ( );

	for ( const auto & element : concatenation.getElements ( ) )
		con->appendElement ( element->template accept < ext::rvalue_ref < regexp::UnboundedRegExpElement < ext::pair < SymbolType, int > > >, GlushkovIndexate::Unbounded < SymbolType > > ( i ) );

	return ext::rvalue_ref < UnboundedRegExpElement < ext::pair < SymbolType, int > > > ( con );
}

template < class SymbolType >
ext::rvalue_ref < regexp::UnboundedRegExpElement < ext::pair < SymbolType, int > > > GlushkovIndexate::Unbounded < SymbolType >::visit(const regexp::UnboundedRegExpIteration < SymbolType > & iteration, int & i) {
	return ext::rvalue_ref < UnboundedRegExpElement < ext::pair < SymbolType, int > > > ( new UnboundedRegExpIteration < ext::pair < SymbolType, int > > ( iteration.getElement ( ).template accept < ext::rvalue_ref < regexp::UnboundedRegExpElement < ext::pair < DefaultSymbolType, int > > >, GlushkovIndexate::Unbounded < SymbolType > > ( i ) ) );
}

template < class SymbolType >
ext::rvalue_ref < regexp::UnboundedRegExpElement < ext::pair < SymbolType, int > > > GlushkovIndexate::Unbounded < SymbolType >::visit(const regexp::UnboundedRegExpSymbol < SymbolType > & symbol, int & i) {
	return ext::rvalue_ref < UnboundedRegExpElement < ext::pair < SymbolType, int > > > ( new UnboundedRegExpSymbol < ext::pair < SymbolType, int > > ( ext::make_pair ( symbol.getSymbol ( ), i++ ) ) );
}

template < class SymbolType >
ext::rvalue_ref < regexp::UnboundedRegExpElement < ext::pair < SymbolType, int > > > GlushkovIndexate::Unbounded < SymbolType >::visit(const regexp::UnboundedRegExpEpsilon < SymbolType > &, int &) {
	return ext::rvalue_ref < UnboundedRegExpElement < ext::pair < SymbolType, int > > > ( new UnboundedRegExpEpsilon < ext::pair < SymbolType, int > > ( ) );
}

template < class SymbolType >
ext::rvalue_ref < regexp::UnboundedRegExpElement < ext::pair < SymbolType, int > > > GlushkovIndexate::Unbounded < SymbolType >::visit(const regexp::UnboundedRegExpEmpty < SymbolType > &, int &) {
	return ext::rvalue_ref < UnboundedRegExpElement < ext::pair < SymbolType, int > > > ( new UnboundedRegExpEmpty < ext::pair < SymbolType, int > > ( ) );
}

} /* namespace regexp */

#endif /* GLUSHKOV_INDEXATE_H_ */
