/*
 * SuffixAutomatonFactors.h
 *
 *  Created on: 10. 1. 2017
 *      Author: Jan Travnicek
 */

#ifndef SUFFIX_AUTOMATON_FACTORS_H_
#define SUFFIX_AUTOMATON_FACTORS_H_

#include <automaton/FSM/DFA.h>
#include <string/LinearString.h>

#include <automaton/run/Run.h>
#

namespace stringology {

namespace query {

/**
 * Query suffix trie for given string.
 *
 * Source: ??
 */

class SuffixAutomatonFactors {
public:
	/**
	 * Query a suffix automaton
	 * @param suffix automaton to query
	 * @param string string to query by
	 * @return occurences of factors
	 */
	template < class SymbolType, class StateType >
	static ext::set < unsigned > query ( const automaton::DFA < SymbolType, StateType > & suffixAutomaton, const string::LinearString < SymbolType > & string );

};

template < class SymbolType, class StateType >
ext::set < unsigned > SuffixAutomatonFactors::query ( const automaton::DFA < SymbolType, StateType > & suffixAutomaton, const string::LinearString < SymbolType > & string ) {
	ext::tuple < bool, StateType, ext::set < unsigned > > run = automaton::run::Run::calculateState ( suffixAutomaton, string );
	if ( ! std::get < 0 > ( run ) )
		return { };

	std::deque < std::pair < StateType, unsigned > > open = { { std::get < 1 > ( run ), 0u } };
	ext::vector < unsigned > tmp;
	unsigned max = 0;
	while ( ! open.empty ( ) ) {
		std::pair < StateType, unsigned > cur = std::move ( open.back ( ) );
		open.pop_back ( );

		if ( suffixAutomaton.getFinalStates ( ).count ( cur.first ) )
			tmp.push_back ( cur.second );

		if ( cur.second > max )
			max = cur.second;

		for ( const auto & transition : suffixAutomaton.getTransitionsFromState ( cur.first ) )
			open.emplace_back ( transition.second, cur.second + 1 );
	}

	unsigned subjectSize = max + string.getContent ( ).size ( );

	ext::set < unsigned > res;
	for ( unsigned dist : tmp )
		res.insert ( subjectSize - dist );

	return res;
}

} /* namespace query */

} /* namespace stringology */

#endif /* SUFFIX_AUTOMATON_FACTORS_H_ */
