#include "FTAtoPDATest.h"

#include <alphabet/BottomOfTheStackSymbol.h>
#include <automaton/TA/DFTA.h>
#include <automaton/TA/NFTA.h>

#include "automaton/convert/ToPostfixPushdownAutomaton.h"

#include <factory/XmlDataFactory.hpp>

#include <label/InitialStateLabel.h>
#include <label/FinalStateLabel.h>

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( FTAtoPDATest, "automaton" );
CPPUNIT_TEST_SUITE_REGISTRATION( FTAtoPDATest );

void FTAtoPDATest::setUp() {
}

void FTAtoPDATest::tearDown() {
}

void FTAtoPDATest::testDFTAtoDPDA() {
	automaton::DFTA < > automaton;

	const common::ranked_symbol < > a ('a', 2);
	const common::ranked_symbol < > b ('b', 1);
	const common::ranked_symbol < > c ('c', 0);
	const ext::set<common::ranked_symbol < > > alphabet {a, b, c};
	automaton.setInputAlphabet(alphabet);

	automaton.addState(DefaultStateType(1));
	automaton.addState(DefaultStateType(2));
	automaton.addState(DefaultStateType(3));

	ext::vector<DefaultStateType> a1States = {DefaultStateType(1), DefaultStateType(3)};
	automaton.addTransition(a, a1States, DefaultStateType(1));
	ext::vector<DefaultStateType> a2States = {DefaultStateType(3), DefaultStateType(3)};
	automaton.addTransition(a, a2States, DefaultStateType(2));
	ext::vector<DefaultStateType> bStates = {DefaultStateType(2)};
	automaton.addTransition(b, bStates, DefaultStateType(1));
	ext::vector<DefaultStateType> cStates;
	automaton.addTransition(c, cStates, DefaultStateType(3));

	automaton.addFinalState(DefaultStateType(3));

	automaton::DPDA < > res = automaton::convert::ToPostfixPushdownAutomaton::convert(automaton);
	alib::XmlDataFactory::toStdout(res);
}

void FTAtoPDATest::testNFTAtoNPDA() {
	automaton::NFTA < > automaton;

	const common::ranked_symbol < > a ('a', 2);
	const common::ranked_symbol < > b ('b', 1);
	const common::ranked_symbol < > c ('c', 0);
	const ext::set<common::ranked_symbol < > > alphabet {a, b, c};
	automaton.setInputAlphabet(alphabet);

	automaton.addState(DefaultStateType(1));
	automaton.addState(DefaultStateType(2));
	automaton.addState(DefaultStateType(3));

	ext::vector<DefaultStateType> a1States = {DefaultStateType(1), DefaultStateType(3)};
	automaton.addTransition(a, a1States, DefaultStateType(1));
	automaton.addTransition(a, a1States, DefaultStateType(3));
	ext::vector<DefaultStateType> a2States = {DefaultStateType(3), DefaultStateType(3)};
	automaton.addTransition(a, a2States, DefaultStateType(2));
	ext::vector<DefaultStateType> bStates = {DefaultStateType(2)};
	automaton.addTransition(b, bStates, DefaultStateType(1));
	ext::vector<DefaultStateType> cStates;
	automaton.addTransition(c, cStates, DefaultStateType(3));

	automaton.addFinalState(DefaultStateType(3));

	automaton::NPDA < > res = automaton::convert::ToPostfixPushdownAutomaton::convert(automaton);
	alib::XmlDataFactory::toStdout(res);
}
