#include "GrammarToGNFTest.h"

#include "grammar/simplify/ToGNF.h"
#include "grammar/generate/GenerateUpToLength.h"

#include "grammar/ContextFree/GNF.h"
#include "grammar/ContextFree/EpsilonFreeCFG.h"

#include <factory/StringDataFactory.hpp>
#include <common/createUnique.hpp>

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( GrammarToGNFTest, "grammar" );
CPPUNIT_TEST_SUITE_REGISTRATION( GrammarToGNFTest );

void GrammarToGNFTest::setUp() {
}

void GrammarToGNFTest::tearDown() {
}

void GrammarToGNFTest::testRemoveToGNFRules1() {
	DefaultSymbolType S = DefaultSymbolType("S");
	DefaultSymbolType A = DefaultSymbolType("A");
	DefaultSymbolType B = DefaultSymbolType("B");
	DefaultSymbolType C = DefaultSymbolType("C");
	DefaultSymbolType D = DefaultSymbolType("D");

	DefaultSymbolType a = DefaultSymbolType("a");
	DefaultSymbolType b = DefaultSymbolType("b");

	grammar::EpsilonFreeCFG < > grammar1(S);
	grammar1.setNonterminalAlphabet({S, A, B, C, D});
	grammar1.setTerminalAlphabet({a, b});

	DefaultSymbolType aprimed = common::createUnique(a, grammar1.getTerminalAlphabet(), grammar1.getNonterminalAlphabet());
	DefaultSymbolType bprimed = common::createUnique(b, grammar1.getTerminalAlphabet(), grammar1.getNonterminalAlphabet());

	grammar::GNF < > grammar2 = grammar::simplify::ToGNF::convert(grammar1);

	grammar::GNF < > grammar3(S);
	grammar3.setNonterminalAlphabet({S, A, B, C, D, aprimed, bprimed});
	grammar3.setTerminalAlphabet({a, b});
	grammar3.addRule(aprimed, ext::make_pair(a, ext::vector<DefaultSymbolType>{}));
	grammar3.addRule(bprimed, ext::make_pair(b, ext::vector<DefaultSymbolType>{}));

	std::cout << alib::StringDataFactory::toString<grammar::Grammar>(grammar::Grammar(grammar2)) << std::endl;
	std::cout << alib::StringDataFactory::toString<grammar::Grammar>(grammar::Grammar(grammar3)) << std::endl;

	CPPUNIT_ASSERT(grammar2 == grammar3);
}

void GrammarToGNFTest::testRemoveToGNFRules2() {
	DefaultSymbolType S = DefaultSymbolType("S");
	DefaultSymbolType A = DefaultSymbolType("A");
	DefaultSymbolType B = DefaultSymbolType("B");
	DefaultSymbolType C = DefaultSymbolType("C");
	DefaultSymbolType D = DefaultSymbolType("D");

	DefaultSymbolType a = DefaultSymbolType("a");
	DefaultSymbolType b = DefaultSymbolType("b");
	DefaultSymbolType c = DefaultSymbolType("c");

	grammar::EpsilonFreeCFG < > grammar1(S);
	grammar1.setNonterminalAlphabet({S, A, B, C, D});
	grammar1.setTerminalAlphabet({a, b, c});
	grammar1.addRule(S, {A});
	grammar1.addRule(A, {A, a});
	grammar1.addRule(A, {A, b});
	grammar1.addRule(A, {c});

	DefaultSymbolType Aprimed = common::createUnique(A, grammar1.getTerminalAlphabet(), grammar1.getNonterminalAlphabet());
	DefaultSymbolType aprimed = common::createUnique(a, grammar1.getTerminalAlphabet(), grammar1.getNonterminalAlphabet());
	DefaultSymbolType bprimed = common::createUnique(b, grammar1.getTerminalAlphabet(), grammar1.getNonterminalAlphabet());
	DefaultSymbolType cprimed = common::createUnique(c, grammar1.getTerminalAlphabet(), grammar1.getNonterminalAlphabet());

	std::cout << alib::StringDataFactory::toString<grammar::Grammar>(grammar::Grammar(grammar1)) << std::endl;

	grammar::GNF < > grammar2 = grammar::simplify::ToGNF::convert(grammar1);

	grammar::GNF < > grammar3(S);
	grammar3.setNonterminalAlphabet({S, A, Aprimed, B, C, D, aprimed, bprimed, cprimed});
	grammar3.setTerminalAlphabet({a, b, c});
	grammar3.addRule(S, ext::make_pair(c, ext::vector<DefaultSymbolType>{Aprimed}));
	grammar3.addRule(S, ext::make_pair(c, ext::vector<DefaultSymbolType>{}));
	grammar3.addRule(A, ext::make_pair(c, ext::vector<DefaultSymbolType>{Aprimed}));
	grammar3.addRule(A, ext::make_pair(c, ext::vector<DefaultSymbolType>{}));
	grammar3.addRule(Aprimed, ext::make_pair(a, ext::vector<DefaultSymbolType>{Aprimed}));
	grammar3.addRule(Aprimed, ext::make_pair(a, ext::vector<DefaultSymbolType>{}));
	grammar3.addRule(Aprimed, ext::make_pair(b, ext::vector<DefaultSymbolType>{Aprimed}));
	grammar3.addRule(Aprimed, ext::make_pair(b, ext::vector<DefaultSymbolType>{}));
	grammar3.addRule(aprimed, ext::make_pair(a, ext::vector<DefaultSymbolType>{}));
	grammar3.addRule(bprimed, ext::make_pair(b, ext::vector<DefaultSymbolType>{}));
	grammar3.addRule(cprimed, ext::make_pair(c, ext::vector<DefaultSymbolType>{}));

	std::cout << alib::StringDataFactory::toString<grammar::Grammar>(grammar::Grammar(grammar2)) << std::endl;
	std::cout << alib::StringDataFactory::toString<grammar::Grammar>(grammar::Grammar(grammar3)) << std::endl;

	CPPUNIT_ASSERT(grammar2 == grammar3);
}

void GrammarToGNFTest::testRemoveToGNFRules3() {
	DefaultSymbolType S = DefaultSymbolType("S");
	DefaultSymbolType A = DefaultSymbolType("A");
	DefaultSymbolType B = DefaultSymbolType("B");
	DefaultSymbolType C = DefaultSymbolType("C");

	DefaultSymbolType a = DefaultSymbolType("a");
	DefaultSymbolType b = DefaultSymbolType("b");
	DefaultSymbolType c = DefaultSymbolType("c");

	grammar::EpsilonFreeCFG < > grammar1(S);
	grammar1.setNonterminalAlphabet({S, A, B, C});
	grammar1.setTerminalAlphabet({a, b, c});
	grammar1.addRule(S, {A});
	grammar1.addRule(A, {B, a});
	grammar1.addRule(B, {A, b});
	grammar1.addRule(A, {c});

	DefaultSymbolType Bprimed = common::createUnique(B, grammar1.getTerminalAlphabet(), grammar1.getNonterminalAlphabet());
	DefaultSymbolType aprimed = common::createUnique(a, grammar1.getTerminalAlphabet(), grammar1.getNonterminalAlphabet());
	DefaultSymbolType bprimed = common::createUnique(b, grammar1.getTerminalAlphabet(), grammar1.getNonterminalAlphabet());
	DefaultSymbolType cprimed = common::createUnique(c, grammar1.getTerminalAlphabet(), grammar1.getNonterminalAlphabet());

	std::cout << alib::StringDataFactory::toString<grammar::Grammar>(grammar::Grammar(grammar1)) << std::endl;

	grammar::GNF < > grammar2 = grammar::simplify::ToGNF::convert(grammar1);

	grammar::GNF < > grammar3(S);
	grammar3.setNonterminalAlphabet({S, A, B, Bprimed, C, aprimed, bprimed, cprimed});
	grammar3.setTerminalAlphabet({a, b, c});
	grammar3.addRule(S, ext::make_pair(c, ext::vector<DefaultSymbolType>{}));
	grammar3.addRule(S, ext::make_pair(c, ext::vector<DefaultSymbolType>{bprimed, Bprimed, aprimed}));
	grammar3.addRule(S, ext::make_pair(c, ext::vector<DefaultSymbolType>{bprimed, aprimed}));
	grammar3.addRule(A, ext::make_pair(c, ext::vector<DefaultSymbolType>{bprimed, aprimed}));
	grammar3.addRule(A, ext::make_pair(c, ext::vector<DefaultSymbolType>{bprimed, Bprimed, aprimed}));
	grammar3.addRule(A, ext::make_pair(c, ext::vector<DefaultSymbolType>{}));
	grammar3.addRule(B, ext::make_pair(c, ext::vector<DefaultSymbolType>{bprimed}));
	grammar3.addRule(B, ext::make_pair(c, ext::vector<DefaultSymbolType>{bprimed, Bprimed}));
	grammar3.addRule(Bprimed, ext::make_pair(a, ext::vector<DefaultSymbolType>{bprimed, Bprimed}));
	grammar3.addRule(Bprimed, ext::make_pair(a, ext::vector<DefaultSymbolType>{bprimed}));
	grammar3.addRule(aprimed, ext::make_pair(a, ext::vector<DefaultSymbolType>{}));
	grammar3.addRule(bprimed, ext::make_pair(b, ext::vector<DefaultSymbolType>{}));
	grammar3.addRule(cprimed, ext::make_pair(c, ext::vector<DefaultSymbolType>{}));

	std::cout << alib::StringDataFactory::toString<grammar::Grammar>(grammar::Grammar(grammar2)) << std::endl;
	std::cout << alib::StringDataFactory::toString<grammar::Grammar>(grammar::Grammar(grammar3)) << std::endl;

	CPPUNIT_ASSERT(grammar2 == grammar3);
}

void GrammarToGNFTest::testRemoveToGNFRules4() {
	DefaultSymbolType A = DefaultSymbolType("A");
	DefaultSymbolType B = DefaultSymbolType("B");
	DefaultSymbolType C = DefaultSymbolType("C");

	DefaultSymbolType a = DefaultSymbolType("a");
	DefaultSymbolType b = DefaultSymbolType("b");

	grammar::EpsilonFreeCFG < > grammar1(A);
	grammar1.setNonterminalAlphabet({A, B, C});
	grammar1.setTerminalAlphabet({a, b});
	grammar1.addRule(A, {B, C});
	grammar1.addRule(A, {a});
	grammar1.addRule(B, {C, A});
	grammar1.addRule(B, {A, b});
	grammar1.addRule(C, {A, B});
	grammar1.addRule(C, {C, C});
	grammar1.addRule(C, {a});

	grammar::GNF < > grammar2 = grammar::simplify::ToGNF::convert(grammar1);

	CPPUNIT_ASSERT(grammar::generate::GenerateUpToLength::generate(grammar1, 7) == grammar::generate::GenerateUpToLength::generate(grammar2, 7));
}
