/*
 * GasTexConverter.cpp
 *
 *  Created on: Jan 1, 2014
 *      Author: Martin Zak
 */

#include "GasTexConverter.h"
#include <registration/AlgoRegistration.hpp>

namespace convert {

void GasTexConverter::printTransitionMap(const ext::map<std::pair<std::string, std::string>, std::string> transitionMap, std::ostream& out) {
	for (const auto& transition : transitionMap) {
		std::string from = transition.first.first;
		std::string to = transition.first.second;
		std::string symbols = transition.second;

		if (from == to) {
			out << "\\drawloop(";
			out << from;
			out << "){";
			out << symbols;
			out << "}\n";

		} else {
			out << "\\drawedge(";
			out << from;
			out << ",";
			out << to;
			out << "){";
			out << symbols;
			out << "}\n";
		}
	}
}

auto GasTexConverterEpsilonNFA = registration::AbstractRegister < GasTexConverter, std::string, const automaton::EpsilonNFA < > & > ( GasTexConverter::convert );
auto GasTexConverterMultiInitialStateNFA = registration::AbstractRegister < GasTexConverter, std::string, const automaton::MultiInitialStateNFA < > & > ( GasTexConverter::convert );
auto GasTexConverterNFA = registration::AbstractRegister < GasTexConverter, std::string, const automaton::NFA < > & > ( GasTexConverter::convert );
auto GasTexConverterDFA = registration::AbstractRegister < GasTexConverter, std::string, const automaton::DFA < > & > ( GasTexConverter::convert );
auto GasTexConverterExtendedNFA = registration::AbstractRegister < GasTexConverter, std::string, const automaton::ExtendedNFA < > & > ( GasTexConverter::convert );
auto GasTexConverterCompactNFA = registration::AbstractRegister < GasTexConverter, std::string, const automaton::CompactNFA < > & > ( GasTexConverter::convert );
auto GasTexConverterNFTA = registration::AbstractRegister < GasTexConverter, std::string, const automaton::NFTA < > & > ( GasTexConverter::convert );
auto GasTexConverterDFTA = registration::AbstractRegister < GasTexConverter, std::string, const automaton::DFTA < > & > ( GasTexConverter::convert );
auto GasTexConverterDPDA = registration::AbstractRegister < GasTexConverter, std::string, const automaton::DPDA < > & > ( GasTexConverter::convert );
auto GasTexConverterSinglePopDPDA = registration::AbstractRegister < GasTexConverter, std::string, const automaton::SinglePopDPDA < > & > ( GasTexConverter::convert );
auto GasTexConverterInputDrivenDPDA = registration::AbstractRegister < GasTexConverter, std::string, const automaton::InputDrivenDPDA < > & > ( GasTexConverter::convert );
auto GasTexConverterInputDrivenNPDA = registration::AbstractRegister < GasTexConverter, std::string, const automaton::InputDrivenNPDA < > & > ( GasTexConverter::convert );
auto GasTexConverterVisiblyPushdownDPDA = registration::AbstractRegister < GasTexConverter, std::string, const automaton::VisiblyPushdownDPDA < > & > ( GasTexConverter::convert );
auto GasTexConverterVisiblyPushdownNPDA = registration::AbstractRegister < GasTexConverter, std::string, const automaton::VisiblyPushdownNPDA < > & > ( GasTexConverter::convert );
auto GasTexConverterRealTimeHeightDeterministicDPDA = registration::AbstractRegister < GasTexConverter, std::string, const automaton::RealTimeHeightDeterministicDPDA < > & > ( GasTexConverter::convert );
auto GasTexConverterRealTimeHeightDeterministicNPDA = registration::AbstractRegister < GasTexConverter, std::string, const automaton::RealTimeHeightDeterministicNPDA < > & > ( GasTexConverter::convert );
auto GasTexConverterNPDA = registration::AbstractRegister < GasTexConverter, std::string, const automaton::NPDA < > & > ( GasTexConverter::convert );
auto GasTexConverterSinglePopNPDA = registration::AbstractRegister < GasTexConverter, std::string, const automaton::SinglePopNPDA < > & > ( GasTexConverter::convert );
auto GasTexConverterOneTapeDTM = registration::AbstractRegister < GasTexConverter, std::string, const automaton::OneTapeDTM < > & > ( GasTexConverter::convert );

} /* namespace convert */
