/*
 * CommonException.h
 *
 * Created on: Apr 1, 2013
 * Author: Martin Zak
 */

#ifndef COMMON_EXCEPTION_H_
#define COMMON_EXCEPTION_H_

#include <exception>
#include <string>
#include <ostream>

#include "../object/ObjectBase.h"

namespace exception {

/**
 * Basic exception from which are derived all other exceptions.
 * Contains reason why the exception occurred.
 */
class CommonException : public std::exception, public alib::ObjectBase {
protected:
	std::string m_cause;
	std::string m_backtrace;
	std::string m_whatMessage;
	std::string m_command;

public:
	explicit CommonException ( std::string cause );

	explicit CommonException ( std::string cause, std::string backtrace, std::string command );

	virtual alib::ObjectBase * clone ( ) const;

	virtual alib::ObjectBase * plunder ( ) &&;

	virtual int compare ( const ObjectBase & other ) const {
		if ( ext::type_index ( typeid ( * this ) ) == ext::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return ext::type_index ( typeid ( * this ) ) - ext::type_index ( typeid ( other ) );
	}

	virtual int compare ( const CommonException & other ) const;

	/**
	 * @return reason why the exception occurred
	 */
	const char * what ( ) const noexcept;

	/**
	 * @return reason why the exception occurred
	 */
	const std::string & getCause ( ) const;

	/**
	 * @return reason why the exception occurred
	 */
	const std::string & getBacktrace ( ) const;

	/**
	 * @return reason why the exception occurred
	 */
	const std::string & getCommand ( ) const;

	virtual void operator >>( std::ostream & os ) const;

	virtual explicit operator std::string ( ) const;

	virtual alib::ObjectBase * inc ( ) &&;
};

} /* namespace exception */

#endif /* COMMON_EXCEPTION_H_ */
