/*
 * RankedSymbol.h
 *
 *  Created on: Apr 10, 2013
 *      Author: Stepan Plachy
 */

#ifndef RANKED_SYMBOL_H_
#define RANKED_SYMBOL_H_

#include "SymbolBase.h"
#include "SymbolFeatures.h"
#include <common/ranked_symbol.hpp>

#include <alphabet/common/SymbolNormalize.h>

#include <core/normalize.hpp>

namespace alphabet {

/**
 * Represents symbol in an alphabet.
 */
template < class SymbolType, class RankType >
class RankedSymbol : public SymbolBase, public common::ranked_symbol < SymbolType, RankType > {
public:
	using common::ranked_symbol < SymbolType, RankType >::ranked_symbol;

	explicit RankedSymbol ( common::ranked_symbol < SymbolType, RankType > symbol ) : common::ranked_symbol < SymbolType, RankType > ( std::move ( symbol ) ) {
	}

	virtual SymbolBase * clone ( ) const;
	virtual SymbolBase * plunder ( ) &&;

	virtual int compare ( const ObjectBase & other ) const {
		if ( ext::type_index ( typeid ( * this ) ) == ext::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return ext::type_index ( typeid ( * this ) ) - ext::type_index ( typeid ( other ) );
	}

	virtual int compare ( const RankedSymbol & other ) const;

	virtual void operator >>( std::ostream & out ) const;

	virtual explicit operator std::string ( ) const;

	using SymbolBase::operator ==;
	using SymbolBase::operator !=;
	using SymbolBase::operator <;
	using SymbolBase::operator <=;
	using SymbolBase::operator >;
	using SymbolBase::operator >=;

	virtual SymbolBase * inc ( ) &&;

	typedef RankedSymbol < > normalized_type;
};

template < class SymbolType, class RankType >
SymbolBase* RankedSymbol < SymbolType, RankType >::clone() const {
	return new RankedSymbol(*this);
}

template < class SymbolType, class RankType >
SymbolBase* RankedSymbol < SymbolType, RankType >::plunder() && {
	return new RankedSymbol(std::move(*this));
}

template < class SymbolType, class RankType >
int RankedSymbol < SymbolType, RankType >::compare(const RankedSymbol& other) const {
	return static_cast < const common::ranked_symbol < SymbolType, RankType > & > ( * this ).compare ( static_cast < const common::ranked_symbol < SymbolType, RankType > & > ( other ) );
}

template < class SymbolType, class RankType >
void RankedSymbol < SymbolType, RankType >::operator>>(std::ostream& out) const {
	out << static_cast < const common::ranked_symbol < SymbolType, RankType > & > ( * this );
}

template < class SymbolType, class RankType >
RankedSymbol < SymbolType, RankType >::operator std::string () const {
	return (std::string) static_cast < const common::ranked_symbol < SymbolType, RankType > & > ( * this );
}

template < class SymbolType, class RankType >
SymbolBase* RankedSymbol < SymbolType, RankType >::inc() && {
	this->m_symbol.inc();
	return NULL;
}

} /* namespace alphabet */

namespace alib {

template < class SymbolType, class RankType >
struct normalize < common::ranked_symbol < SymbolType, RankType >, typename std::enable_if < std::is_same < common::ranked_symbol < SymbolType, RankType >, common::ranked_symbol < > >::value >::type > {
	static common::ranked_symbol < > && eval ( common::ranked_symbol < SymbolType, RankType > && value ) {
		return std::move ( value );
	}
};

template < class SymbolType, class RankType >
struct normalize < common::ranked_symbol < SymbolType, RankType >, typename std::enable_if < ! std::is_same < common::ranked_symbol < SymbolType, RankType >, common::ranked_symbol < > >::value >::type > {
	static common::ranked_symbol < > eval ( common::ranked_symbol < SymbolType, RankType > && value ) {
		return alphabet::SymbolNormalize::normalizeRankedSymbol ( std::move ( value ) );
	}
};

template < class SymbolType, class RankType >
struct normalize < alphabet::RankedSymbol < SymbolType, RankType >, typename std::enable_if < ! std::is_same < alphabet::RankedSymbol < SymbolType, RankType >, alphabet::RankedSymbol < > >::value >::type > {
	static alphabet::RankedSymbol < > eval ( alphabet::RankedSymbol < SymbolType, RankType > && value ) {
		return alphabet::RankedSymbol < > ( normalize < common::ranked_symbol < SymbolType, RankType > >::eval ( std::move ( value ) ) );
	}
};

} /* namespace alib */

#endif /* RANKED_SYMBOL_H_ */
