/*
 * VariablesBarSymbol.h
 *
 *  Created on: Apr 10, 2013
 *      Author: Jan Travnicek
 */

#ifndef VARIABLES_BAR_SYMBOL_H_
#define VARIABLES_BAR_SYMBOL_H_

#include "SymbolBase.h"
#include "Symbol.h"
#include "RankedSymbol.h"
#include <object/Object.h>

namespace alphabet {

/**
 * Represents bar symbol for tree linearization.
 */
class VariablesBarSymbol : public SymbolBase {
public:
	/**
	 * Creates a bar symbol.
	 * @param symbol name of the symbol
	 */
	explicit VariablesBarSymbol ( );

	virtual SymbolBase * clone ( ) const;
	virtual SymbolBase * plunder ( ) &&;

	virtual int compare ( const ObjectBase & other ) const {
		if ( ext::type_index ( typeid ( * this ) ) == ext::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return ext::type_index ( typeid ( * this ) ) - ext::type_index ( typeid ( other ) );
	}

	virtual int compare ( const VariablesBarSymbol & other ) const;

	virtual void operator >>( std::ostream & out ) const;

	virtual explicit operator std::string ( ) const;

	template < typename Base >
	static inline typename std::enable_if <   std::is_integral < Base >::value, Base >::type instance ( );
	template < typename Base >
	static inline typename std::enable_if < ! std::is_integral < Base >::value, Base >::type instance ( );

	virtual SymbolBase * inc ( ) &&;

	typedef VariablesBarSymbol normalized_type;
};

template < typename Base >
inline typename std::enable_if < std::is_integral < Base >::value, Base >::type VariablesBarSymbol::instance ( ) {
	return '!';
}

template < >
inline std::string VariablesBarSymbol::instance < std::string > ( ) {
	return std::string ( 1, VariablesBarSymbol::instance < char > ( ) );
}

template < >
inline VariablesBarSymbol VariablesBarSymbol::instance < VariablesBarSymbol > ( ) {
	return VariablesBarSymbol ( );
}

template < >
inline Symbol VariablesBarSymbol::instance < Symbol > ( ) {
	return Symbol ( VariablesBarSymbol ( ) );
}

// TODO make partially specialised when needed by classes or variables, functions can be partially specialsed
template < >
inline common::ranked_symbol < > VariablesBarSymbol::instance < common::ranked_symbol < > > ( ) {
	return common::ranked_symbol < > ( DefaultSymbolType ( VariablesBarSymbol ( ) ), 0);
}

template < >
inline RankedSymbol < > VariablesBarSymbol::instance < RankedSymbol < > > ( ) {
	return RankedSymbol < > ( VariablesBarSymbol::instance < common::ranked_symbol < > > ( ) );
}

template < >
inline alib::Object VariablesBarSymbol::instance < alib::Object > ( ) {
	return alib::Object ( VariablesBarSymbol ( ) );
}

} /* namespace alphabet */

#endif /* VARIABLES_BAR_SYMBOL_H_ */
