/*
 * RankedSymbol.h
 *
 *  Created on: Apr 10, 2013
 *      Author: Stepan Plachy
 */

#ifndef _XML_RANKED_SYMBOL_H_
#define _XML_RANKED_SYMBOL_H_

#include <alphabet/RankedSymbol.h>
#include <core/xmlApi.hpp>
#include <primitive/xml/Unsigned.h>

namespace alib {

template < typename T, typename R >
struct xmlApi < common::ranked_symbol < T, R > > {
	static common::ranked_symbol < T, R > parse ( ext::deque < sax::Token >::iterator & input );
	static bool first ( const ext::deque < sax::Token >::const_iterator & input );
	static const std::string & xmlTagName ( );
	static void compose ( ext::deque < sax::Token > & output, const common::ranked_symbol < T, R > & data );
};

template < typename T, typename R >
common::ranked_symbol < T, R > xmlApi < common::ranked_symbol < T, R > >::parse ( ext::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, xmlTagName ( ) );
	T data = alib::xmlApi < T >::parse ( input );
	R rank = alib::xmlApi < R >::parse ( input );
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, xmlTagName ( ) );
	return common::ranked_symbol < T, R > ( std::move ( data ), std::move ( rank ) );
}

template < typename T, typename R >
bool xmlApi < common::ranked_symbol < T, R > >::first ( const ext::deque < sax::Token >::const_iterator & input ) {
	return sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, xmlTagName( ) );
}

template < typename T, typename R >
const std::string & xmlApi < common::ranked_symbol < T, R > >::xmlTagName ( ) {
	static std::string xmlTagName = "RankedSymbol";

	return xmlTagName;
}

template < typename T, typename R >
void xmlApi < common::ranked_symbol < T, R > >::compose ( ext::deque < sax::Token > & output, const common::ranked_symbol < T, R > & input ) {
	output.emplace_back ( xmlTagName ( ), sax::Token::TokenType::START_ELEMENT );
	alib::xmlApi < T >::compose ( output, input.getSymbol ( ) );
	alib::xmlApi < R >::compose ( output, input.getRank ( ) );
	output.emplace_back ( xmlTagName ( ), sax::Token::TokenType::END_ELEMENT );
}

template < typename T, typename R >
struct xmlApi < alphabet::RankedSymbol < T, R > > {
	static alphabet::RankedSymbol < T, R > parse ( ext::deque < sax::Token >::iterator & input );
	static bool first ( const ext::deque < sax::Token >::const_iterator & input );
	static const std::string & xmlTagName ( );
	static void compose ( ext::deque < sax::Token > & output, const alphabet::RankedSymbol < T, R > & data );
};

template < typename T, typename R >
alphabet::RankedSymbol < T, R > xmlApi < alphabet::RankedSymbol < T, R > >::parse ( ext::deque < sax::Token >::iterator & input ) {
	return alphabet::RankedSymbol < T, R > ( xmlApi < common::ranked_symbol < T, R > >::parse ( input ) );
}

template < typename T, typename R >
bool xmlApi < alphabet::RankedSymbol < T, R > >::first ( const ext::deque < sax::Token >::const_iterator & input ) {
	return xmlApi < common::ranked_symbol < T, R > >::first ( input );
}

template < typename T, typename R >
const std::string & xmlApi < alphabet::RankedSymbol < T, R > >::xmlTagName ( ) {
	return xmlApi < common::ranked_symbol < T, R > >::xmlTagName ( );
}

template < typename T, typename R >
void xmlApi < alphabet::RankedSymbol < T, R > >::compose ( ext::deque < sax::Token > & output, const alphabet::RankedSymbol < T, R > & input ) {
	xmlApi < common::ranked_symbol < T, R > >::compose ( output, input );
}

} /* namespace alib */

#endif /* _XML_RANKED_SYMBOL_H_ */
