/*
 * FinalStateLabel.cpp
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travicek
 */

#include "FinalStateLabel.h"
#include <sax/FromXMLParserHelper.h>
#include "Label.h"
#include <object/Object.h>
#include "UniqueLabel.h"

#include <registration/ValuePrinterRegistration.hpp>
#include <registration/XmlRegistration.hpp>

namespace label {

FinalStateLabel::FinalStateLabel() {
}

LabelBase* FinalStateLabel::clone() const {
	return new FinalStateLabel(*this);
}

LabelBase* FinalStateLabel::plunder() && {
	return new FinalStateLabel(std::move(*this));
}

int FinalStateLabel::compare(const FinalStateLabel&) const {
	return 0;
}

void FinalStateLabel::operator>>(std::ostream& out) const {
	out << "(FinalStateLabel)";
}

FinalStateLabel::operator std::string() const {
	return FinalStateLabel::instance < std::string > ( );
}

FinalStateLabel FinalStateLabel::parse(ext::deque<sax::Token>::iterator& input) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, FinalStateLabel::getXmlTagName());
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, FinalStateLabel::getXmlTagName());
	return FinalStateLabel();
}

void FinalStateLabel::compose(ext::deque<sax::Token>& out, const FinalStateLabel & ) {
	out.emplace_back(FinalStateLabel::getXmlTagName(), sax::Token::TokenType::START_ELEMENT);
	out.emplace_back(FinalStateLabel::getXmlTagName(), sax::Token::TokenType::END_ELEMENT);
}

LabelBase* FinalStateLabel::inc() && {
	return new UniqueLabel(Label(std::move(*this)), primitive::Integer(0));
}

} /* namespace label */

namespace {

static auto valuePrinter = registration::ValuePrinterRegister < label::FinalStateLabel > ( );

static auto xmlWrite = registration::XmlWriterRegister < label::Label, label::FinalStateLabel > ( );
static auto xmlWrite2 = registration::XmlWriterRegister < alib::Object, label::FinalStateLabel > ( );

static auto xmlReader = registration::XmlReaderRegister < label::Label, label::FinalStateLabel > ( );
static auto xmlReaded2 = registration::XmlReaderRegister < alib::Object, label::FinalStateLabel > ( );

} /* namespace */
