/*
 * HexavigesimalLabel.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef HEXAVIGESIMAL_LABEL_H_
#define HEXAVIGESIMAL_LABEL_H_

#include <hexavigesimal>

#include "LabelBase.h"

namespace label {

/**
 * Represents symbol in an alphabet.
 */
class HexavigesimalLabel final : public LabelBase {
protected:
	int m_hexavigesimal;

public:
	/**
	 * Creates new symbol with given name.
	 * @param symbol name of the symbol
	 */
	explicit HexavigesimalLabel ( int hexavigesimal );

	virtual LabelBase * clone ( ) const;

	virtual LabelBase * plunder ( ) &&;

	/**
	 * @return name of the symbol
	 */
	int getData ( ) const;

	void setData ( int data );

	virtual int compare ( const ObjectBase & other ) const {
		if ( ext::type_index ( typeid ( * this ) ) == ext::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return ext::type_index ( typeid ( * this ) ) - ext::type_index ( typeid ( other ) );
	}

	virtual int compare ( const HexavigesimalLabel & other ) const;

	virtual void operator >>( std::ostream & ) const;

	virtual explicit operator std::string ( ) const;

	static const std::string & getXmlTagName() {
		static std::string xmlTagName = "HexavigesimalLabel";

		return xmlTagName;
	}

	static HexavigesimalLabel parse ( ext::deque < sax::Token >::iterator & input );

	static void compose ( ext::deque < sax::Token > & out, const HexavigesimalLabel & label );

	virtual LabelBase * inc ( ) &&;

	typedef HexavigesimalLabel normalized_type;
};

} /* namespace label */

#endif /* HEXAVIGESIMAL_LABEL_H_ */
