/*
 * ObjectsSet.h
 *
 * Created on: Sep 27, 2017
 * Author: Jan Travnicek
 */

#ifndef _STRING_OBJECTS_SET_H_
#define _STRING_OBJECTS_SET_H_

#include <container/ObjectsSet.h>
#include <core/stringApi.hpp>

#include <container/ContainerFromStringLexer.h>

namespace alib {

template<class ValueType >
struct stringApi < ext::set < ValueType > > {
	static ext::set < ValueType > parse ( std::istream & input );
	static bool first ( std::istream & input );
	static void compose ( std::ostream & output, const ext::set < ValueType > & container );
};

template<class ValueType >
ext::set < ValueType > stringApi < ext::set < ValueType > >::parse ( std::istream & input ) {
	container::ContainerFromStringLexer::Token token = container::ContainerFromStringLexer::next ( input );
	if(token.type != container::ContainerFromStringLexer::TokenType::SET_BEGIN)
		throw exception::CommonException("Expected SET_BEGIN token.");

	token = container::ContainerFromStringLexer::next ( input );

	ext::set<ValueType> objectsSet;
	if(token.type != container::ContainerFromStringLexer::TokenType::SET_END) while(true) {
		container::ContainerFromStringLexer::putback(input, std::move(token));
		ValueType innerObject = stringApi < ValueType >::parse ( input );
		objectsSet.insert ( std::move ( innerObject ) );

		token = container::ContainerFromStringLexer::next(input);
		if(token.type != container::ContainerFromStringLexer::TokenType::COMMA)
			break;

		token = container::ContainerFromStringLexer::next(input);
	}

	if(token.type != container::ContainerFromStringLexer::TokenType::SET_END)
		throw exception::CommonException("Expected SET_END token.");
	return objectsSet;
}

template<class ValueType >
bool stringApi < ext::set < ValueType > >::first ( std::istream & input ) {
	container::ContainerFromStringLexer::Token token = container::ContainerFromStringLexer::next ( input );
	bool res = token.type == container::ContainerFromStringLexer::TokenType::SET_BEGIN;
	container::ContainerFromStringLexer::putback ( input, token );
	return res;
}

template<class ValueType >
void stringApi < ext::set < ValueType > >::compose ( std::ostream & output, const ext::set < ValueType > & container ) {
	output << '{';
	bool first = true;
	for(const ValueType & innerObject : container) {
		if(!first)
			output << ", ";
		else
			first = false;
		stringApi < ValueType >::compose ( output, innerObject );
	}
	output << '}';
}

template<class ValueType >
struct stringApi < container::ObjectsSet < ValueType > > {
	static container::ObjectsSet < ValueType > parse ( std::istream & input );
	static bool first ( std::istream & input );
	static void compose ( std::ostream & output, const container::ObjectsSet < ValueType > & container );
};

template<class ValueType >
container::ObjectsSet < ValueType > stringApi < container::ObjectsSet < ValueType > >::parse ( std::istream & input ) {
	return container::ObjectsSet < ValueType > ( stringApi < ext::set < ValueType > >::parse ( input ) );
}

template<class ValueType >
bool stringApi < container::ObjectsSet < ValueType > >::first ( std::istream & input ) {
	return stringApi < ext::set < ValueType > >::first ( input );
}

template<class ValueType >
void stringApi < container::ObjectsSet < ValueType > >::compose ( std::ostream & output, const container::ObjectsSet < ValueType > & container ) {
	stringApi < ext::set < ValueType > >::compose ( output, container );
}

} /* namespace alib */

#endif /* _STRING_OBJECTS_SET_H_ */
