/*
 * GrammarToStringComposerCommon.h
 *
 * Created on: Sep 27, 2017
 * Author: Jan Travnicek
 */

#ifndef GRAMMAR_TO_STRING_COMPOSER_COMMON_H_
#define GRAMMAR_TO_STRING_COMPOSER_COMMON_H_

#include <ostream>

#include <grammar/GrammarClasses.h>
#include <grammar/Grammar.h>

#include <core/stringApi.hpp>

namespace grammar {

class GrammarToStringComposerCommon {
public:
	template < class T, class SymbolType = typename grammar::SymbolTypeOfGrammar < T > >
	static void composeCFLikeGrammar(std::ostream& output, const T& grammar);
	template < class T, class SymbolType = typename grammar::SymbolTypeOfGrammar < T > >
	static void composeCSLikeGrammar(std::ostream& output, const T& grammar);
	template < class T, class SymbolType = typename grammar::SymbolTypeOfGrammar < T > >
	static void composePreservingCSLikeGrammar(std::ostream& output, const T& grammar);
};

template < class T, class SymbolType >
void GrammarToStringComposerCommon::composeCFLikeGrammar(std::ostream& output, const T& grammar) {
	bool first;

	output << " (" << std::endl;

	output << "{";
	first = false;
	for(const auto& symbol : grammar.getNonterminalAlphabet() ) {
		if(first)
			output << ", ";
		else
			first = true;
		alib::stringApi < SymbolType >::compose(output, symbol);
	}
	output << "}," << std::endl;
	output << "{";
	first = false;
	for(const auto& symbol : grammar.getTerminalAlphabet() ) {
		if(first)
			output << ", ";
		else
			first = true;
		alib::stringApi < SymbolType >::compose(output, symbol);
	}
	output << "}," << std::endl;
	output << "{ ";
	first = true;
	for(const auto& rule : grammar.getRawRules() ) {
		if(first)
			first = false;
		else
			output << "," << std::endl << "  ";
		alib::stringApi < SymbolType >::compose(output, rule.first);
		output << " ->";
		bool innerFirst = true;
		for(const auto& rhs : rule.second) {
			if(innerFirst)
				innerFirst = false;
			else
				output << " |";
			for(const auto& symbol : rhs) {
				output << " ";
				alib::stringApi < SymbolType >::compose(output, symbol);
			}
		}
	}
	output << "}," << std::endl;
	alib::stringApi < SymbolType >::compose(output, grammar.getInitialSymbol());
	output << ")" << std::endl;
}

template < class T, class SymbolType >
void GrammarToStringComposerCommon::composeCSLikeGrammar(std::ostream& output, const T& grammar) {
	bool first;

	output << " (" << std::endl;

	output << "{";
	first = false;
	for(const auto& symbol : grammar.getNonterminalAlphabet() ) {
		if(first)
			output << ", ";
		else
			first = true;
		alib::stringApi < SymbolType >::compose(output, symbol);
	}
	output << "}," << std::endl;
	output << "{";
	first = false;
	for(const auto& symbol : grammar.getTerminalAlphabet() ) {
		if(first)
			output << ", ";
		else
			first = true;
		alib::stringApi < SymbolType >::compose(output, symbol);
	}
	output << "}," << std::endl;
	output << "{";
	first = true;
	for(const auto& rule : grammar.getRules() ) {
		if(first)
			first = false;
		else
			output << "," << std::endl << " ";
		for(const auto& symbol : rule.first) {
			output << " ";
			alib::stringApi < SymbolType >::compose(output, symbol);
		}
		output << " ->";
		bool innerFirst = true;
		for(const auto& rhs : rule.second) {
			if(innerFirst)
				innerFirst = false;
			else
				output << " |";
			for(const auto& symbol : rhs) {
				output << " ";
				alib::stringApi < SymbolType >::compose(output, symbol);
			}
		}
	}
	output << "}," << std::endl;
	alib::stringApi < SymbolType >::compose(output, grammar.getInitialSymbol());
	output << ")" << std::endl;
}

template < class T, class SymbolType >
void GrammarToStringComposerCommon::composePreservingCSLikeGrammar(std::ostream& output, const T& grammar) {
	bool first;

	output << " (" << std::endl;

	output << "{";
	first = false;
	for(const auto& symbol : grammar.getNonterminalAlphabet() ) {
		if(first)
			output << ", ";
		else
			first = true;
		alib::stringApi < SymbolType >::compose(output, symbol);
	}
	output << "}," << std::endl;
	output << "{";
	first = false;
	for(const auto& symbol : grammar.getTerminalAlphabet() ) {
		if(first)
			output << ", ";
		else
			first = true;
		alib::stringApi < SymbolType >::compose(output, symbol);
	}
	output << "}," << std::endl;
	output << "{";
	first = true;
	for(const auto& rule : grammar.getRules() ) {
		if(first)
			first = false;
		else
			output << "," << std::endl << " ";
		for(const auto& symbol : std::get<0>(rule.first)) {
			output << " ";
			alib::stringApi < SymbolType >::compose(output, symbol);
		}
		output << " | ";
		alib::stringApi < SymbolType >::compose(output, std::get<1>(rule.first));
		output << " |";
		for(const auto& symbol : std::get<2>(rule.first)) {
			output << " ";
			alib::stringApi < SymbolType >::compose(output, symbol);
		}
		output << " ->";
		bool innerFirst = true;
		for(const auto& rhs : rule.second) {
			if(innerFirst)
				innerFirst = false;
			else
				output << " |";
			for(const auto& symbol : rhs) {
				output << " ";
				alib::stringApi < SymbolType >::compose(output, symbol);
			}
		}
	}
	output << "}," << std::endl;
	alib::stringApi < SymbolType >::compose(output, grammar.getInitialSymbol());
	output << ")" << std::endl;
}

} /* namespace grammar */

#endif /* GRAMMAR_TO_STRING_COMPOSER_COMMON_H_ */
