/*
 * Map.h
 *
 * Created on: Apr 1, 2013
 * Author: Jan Travnicek
 */

#ifndef _XML_OBJECTS_MAP_H_
#define _XML_OBJECTS_MAP_H_

#include <container/ObjectsMap.h>
#include <core/xmlApi.hpp>
#include <container/xml/ObjectsPair.h>

namespace alib {

template < typename T, typename R >
struct xmlApi < ext::map < T, R > > {
	static ext::map < T, R > parse ( ext::deque < sax::Token >::iterator & input );
	static bool first ( const ext::deque < sax::Token >::const_iterator & input );
	static const std::string & xmlTagName ( );
	static void compose ( ext::deque < sax::Token > & output, const ext::map < T, R > & data );
};

template < typename T, typename R >
ext::map < T, R > xmlApi < ext::map < T, R > >::parse ( ext::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, xmlTagName ( ) );

	ext::map < T, R > map;

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) )
		map.insert ( alib::xmlApi < ext::pair < T, R > >::parse ( input ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, xmlTagName ( ) );
	return map;
}

template < typename T, typename R >
bool xmlApi < ext::map < T, R > >::first ( const ext::deque < sax::Token >::const_iterator & input ) {
	return sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, xmlTagName ( ) );
}

template < typename T, typename R >
const std::string & xmlApi < ext::map < T, R > >::xmlTagName ( ) {
	static std::string xmlTagName = "Map";

	return xmlTagName;
}

template < typename T, typename R >
void xmlApi < ext::map < T, R > >::compose ( ext::deque < sax::Token > & output, const ext::map < T, R > & input ) {
	output.emplace_back ( xmlTagName ( ), sax::Token::TokenType::START_ELEMENT );

	for ( const std::pair < const T, R > & item : input )
		alib::xmlApi < ext::pair < const T, R > >::compose ( output, item );

	output.emplace_back ( xmlTagName ( ), sax::Token::TokenType::END_ELEMENT );
}

template < typename T, typename R >
struct xmlApi < container::ObjectsMap < T, R > > {
	static container::ObjectsMap < T, R > parse ( ext::deque < sax::Token >::iterator & input );
	static bool first ( const ext::deque < sax::Token >::const_iterator & input );
	static const std::string & xmlTagName ( );
	static void compose ( ext::deque < sax::Token > & output, const container::ObjectsMap < T, R > & data );
};

template < typename T, typename R >
container::ObjectsMap < T, R > xmlApi < container::ObjectsMap < T, R > >::parse ( ext::deque < sax::Token >::iterator & input ) {
	return container::ObjectsMap < T, R > ( xmlApi < ext::map < T, R > >::parse ( input ) );
}

template < typename T, typename R >
bool xmlApi < container::ObjectsMap < T, R > >::first ( const ext::deque < sax::Token >::const_iterator & input ) {
	return sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, xmlTagName ( ) );
}

template < typename T, typename R >
const std::string & xmlApi < container::ObjectsMap < T, R > >::xmlTagName ( ) {
	return xmlApi < ext::map < T, R > >::xmlTagName ( );
}

template < typename T, typename R >
void xmlApi < container::ObjectsMap < T, R > >::compose ( ext::deque < sax::Token > & output, const container::ObjectsMap < T, R > & input ) {
	xmlApi < ext::map < T, R > >::compose ( output, input );
}

} /* namespace alib */

#endif /* _XML_OBJECTS_MAP_H_ */
