/*
 * ObjectsTree.h
 *
 * Created on: Apr 1, 2013
 * Author: Jan Travnicek
 */

#ifndef _XML_OBJECTS_TREE_H_
#define _XML_OBJECTS_TREE_H_

#include <container/ObjectsTree.h>
#include <core/xmlApi.hpp>

namespace alib {

template < typename T >
struct xmlApi < ext::tree < T > > {
	static ext::tree < T > parse ( ext::deque < sax::Token >::iterator & input );
	static bool first ( const ext::deque < sax::Token >::const_iterator & input );
	static const std::string & xmlTagName ( );
	static void compose ( ext::deque < sax::Token > & output, const ext::tree < T > & data );
};

template < typename T >
ext::tree < T > xmlApi < ext::tree < T > >::parse ( ext::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, xmlTagName ( ) );
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "Root" );

	ext::tree < T > tree ( alib::xmlApi < T >::parse ( input ) );

	typename ext::tree < T >::const_children_iterator under = tree.root ( );

	unsigned level = 0;

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) ) {
		if ( sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, "Children" ) ) {
			sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "Children" );

			if ( level > 0 )
				under = std::prev ( under->end ( ) );

			++level;
		}

		tree.push_back ( under, alib::xmlApi < T >::parse ( input ) );

		while ( sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::END_ELEMENT, "Children" ) ) {
			sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "Children" );

			under = under->parent ( );
			--level;
		}
	}

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "Root" );
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, xmlTagName ( ) );
	return tree;
}

template < typename T >
bool xmlApi < ext::tree < T > >::first ( const ext::deque < sax::Token >::const_iterator & input ) {
	return sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, xmlTagName ( ) );
}

template < typename T >
const std::string & xmlApi < ext::tree < T > >::xmlTagName ( ) {
	static std::string xmlTagName = "Tree";

	return xmlTagName;
}

template < typename T >
void xmlApi < ext::tree < T > >::compose ( ext::deque < sax::Token > & output, const ext::tree < T > & input ) {
	output.emplace_back ( xmlTagName ( ), sax::Token::TokenType::START_ELEMENT );
	output.emplace_back ( "Root", sax::Token::TokenType::START_ELEMENT );

	unsigned level = 0;

	for ( typename ext::tree < T >::const_prefix_iterator iter = input.prefix_begin ( ); iter != input.prefix_end ( ); ) {
		while ( iter.getLevel ( ) > level ) {
			output.emplace_back ( "Children", sax::Token::TokenType::START_ELEMENT );
			++level;
		}

		alib::xmlApi < T >::compose ( output, * iter );
		++iter;

		while ( iter.getLevel ( ) < level ) {
			output.emplace_back ( "Children", sax::Token::TokenType::END_ELEMENT );
			--level;
		}
	}

	output.emplace_back ( "Root", sax::Token::TokenType::END_ELEMENT );
	output.emplace_back ( xmlTagName ( ), sax::Token::TokenType::END_ELEMENT );
}

template < typename T >
struct xmlApi < container::ObjectsTree < T > > {
	static container::ObjectsTree < T > parse ( ext::deque < sax::Token >::iterator & input );
	static bool first ( const ext::deque < sax::Token >::const_iterator & input );
	static const std::string & xmlTagName ( );
	static void compose ( ext::deque < sax::Token > & output, const container::ObjectsTree < T > & data );
};

template < typename T >
container::ObjectsTree < T > xmlApi < container::ObjectsTree < T > >::parse ( ext::deque < sax::Token >::iterator & input ) {
	return container::ObjectsTree < T > ( xmlApi < ext::tree < T > >::parse ( input ) );
}

template < typename T >
bool xmlApi < container::ObjectsTree < T > >::first ( const ext::deque < sax::Token >::const_iterator & input ) {
	return xmlApi < ext::tree < T > >::first ( input );
}

template < typename T >
const std::string & xmlApi < container::ObjectsTree < T > >::xmlTagName ( ) {
	return xmlApi < ext::tree < T > >::xmlTagName ( );
}

template < typename T >
void xmlApi < container::ObjectsTree < T > >::compose ( ext::deque < sax::Token > & output, const container::ObjectsTree < T > & input ) {
	xmlApi < ext::tree < T > >::compose ( output, input );
}

} /* namespace alib */

#endif /* _XML_OBJECTS_TREE_H_ */
