/*
 * FromXMLParserHelper.cpp
 *
 *  Created on: Oct 12, 2013
 *      Author: Jan Travnicek
 */

#include "FromXMLParserHelper.h"
#include "ParserException.h"
#include <exception/CommonException.h>
#include "SaxParseInterface.h"

namespace sax {

void FromXMLParserHelper::skipAttributes ( ext::deque < Token >::const_iterator & input, Token::TokenType type ) {
	if ( type == sax::Token::TokenType::START_ELEMENT || type == sax::Token::TokenType::END_ELEMENT || type == sax::Token::TokenType::CHARACTER ) {
		while ( input->getType ( ) == sax::Token::TokenType::START_ATTRIBUTE ) {
			input += 3; /* skip START, PAYLOAD, END */
		}
	}
}

void FromXMLParserHelper::skipAttributes ( ext::deque < Token >::iterator & input, Token::TokenType type ) {
	if ( type == sax::Token::TokenType::START_ELEMENT || type == sax::Token::TokenType::END_ELEMENT || type == sax::Token::TokenType::CHARACTER ) {
		while ( input->getType ( ) == sax::Token::TokenType::START_ATTRIBUTE ) {
			input += 3; /* skip START, PAYLOAD, END */
		}
	}
}

bool FromXMLParserHelper::isToken(ext::deque<Token>::const_iterator input, Token::TokenType type, const std::string& data) {
	skipAttributes ( input, type );
	return input->getType() == type && input->getData() == data;
}

bool FromXMLParserHelper::isTokenType(ext::deque<Token>::const_iterator input, Token::TokenType type) {
	skipAttributes ( input, type );
	return input->getType() == type;
}

void FromXMLParserHelper::popToken(ext::deque<Token>::iterator& input, Token::TokenType type, const std::string& data) {
	if (isToken(input, type, data)) {
		skipAttributes ( input, type );
		++input;
	} else {
		throw ParserException(Token(data, type), *input);
	}
}

std::string FromXMLParserHelper::popTokenData(ext::deque<Token>::iterator& input, Token::TokenType type) {
	if(isTokenType(input, type)) {
		skipAttributes ( input, type );
		std::string result = std::move(*input).moveData();
		++input;
		return result;
	} else {
		throw ParserException(Token("?", type), *input);
	}
}

std::string FromXMLParserHelper::getTokenData(ext::deque<Token>::const_iterator input, Token::TokenType type) {
	if(isTokenType(input, type)) {
		skipAttributes ( input, type );
		return input->getData();
	} else {
		throw ParserException(Token("?", type), *input);
	}
}

ext::deque< sax::Token > FromXMLParserHelper::parseInput(const std::string& input) {
	ext::deque < sax::Token > res;
	sax::SaxParseInterface::parseFile ( input, res );
	return res;
}

ext::deque < ext::deque < sax::Token > > FromXMLParserHelper::parseInput(bool needSingle, /*const*/ TCLAP::MultiArg < std::string >& input) {
	ext::deque < ext::deque < sax::Token > > inputTokens;

	if ( input.isSet ( ) ) {
		if(needSingle && input.getValue().size() > 1)
			throw exception::CommonException("Multiple parameters when single required.");

		for ( const std::string & fileName : input.getValue() ) {
			inputTokens.emplace_back ( parseInput ( fileName ) );
		}
	} else {
		inputTokens.emplace_back ( parseInput ( "-" ) );
	}

	return inputTokens;
}

ext::deque < sax::Token > FromXMLParserHelper::parseInput(/*const*/ TCLAP::ValueArg < std::string >& input) {
	if ( input.isSet ( ) ) {
		return parseInput ( input.getValue() );
	} else {
		return parseInput ( "-" );
	}
}
} /* namespace sax */
