/*
 * SaxParseInterface.h
 *
 *  Created on: 8.8.2012
 *      Author: Martin Zak
 */

#ifndef SAX_PARSE_INTERFACE_H_
#define SAX_PARSE_INTERFACE_H_

#include <libxml/xmlreader.h>
#include <deque>
#include "Token.h"

namespace sax {

/**
 * This class performs parsing of file or string containing XML. Contains callback
 * methods for libxml SAX parser.
 */
class SaxParseInterface {
	static int xmlSAXUserParse(xmlTextReaderPtr writer, ext::deque<Token>& out);
public:
	/**
	 * Parses the string containing XML.
	 * @param xmlIn input XML
	 * @param out list of tokens that are returned
	 * @throws CommonException when an error occurs (e.g. XML is not valid)
	 */
	static void parseMemory(const std::string& xmlIn, ext::deque<Token>& out);

	/**
	 * Parses the file containing XML.
	 * @param filename input XML
	 * @param out list of tokens that are returned
	 * @throws CommonException when an error occurs (e.g. file doesn't exist, XML is not valid)
	 */
	static void parseFile(const std::string& filename, ext::deque<Token>& out);
	
	/**
	 * Parses the XML from stdin.
	 * @param out list of tokens that are returned
	 */
	static void parseStdin(ext::deque<Token>& out);
	
	/**
	 * Parses the XML from stream.
	 * @param in input XML
	 * @param out list of tokens that are returned
	 */
	static void parseStream(std::istream& in, ext::deque<Token>& out);
};

} /* namespace sax */

#endif /* SAX_PARSE_INTERFACE_H_ */
