/*
 * Prompt.cpp
 *
 *  Created on: 20. 7. 2017
 *	  Author: Jan Travnicek
 */

#include "Prompt.h"

#include <cstdlib>
#include <cctype>
#include <cstring>

#include <readline/readline.h>
#include <readline/history.h>

#include <lexer/Lexer.h>
#include <parser/Parser.h>

#include <factory/XmlDataFactory.hpp>
#include <global/GlobalData.h>

char * Prompt::stripwhite ( char * begin ) {
	while ( isspace ( * begin ) )
		++ begin;

	char * end = begin + strlen ( begin ) - 1;
	while ( end >= begin && isspace ( * end ) )
		-- end;

	if ( end >= begin )
		* ++ end = '\0';

	return begin;
}

Prompt::Prompt ( cli::Environment environment ) : m_prefix ( "> " ), m_history_file ( std::string ( std::getenv ( "HOME" ) ) + "/.aql_history" ), m_environment ( std::move ( environment ) ) {
	read_history ( m_history_file.c_str ( ) );
}

Prompt::~Prompt ( ) {
	write_history ( m_history_file.c_str ( ) );
}

cli::Command::Result Prompt::run ( ) {
	cli::Command::Result state = cli::Command::Result::OK;

	while ( state != cli::Command::Result::QUIT && state != cli::Command::Result::ERROR ) {
		char * line = readline ( m_prefix.c_str ( ) );

		if ( ! line )
			continue;

		char * s = stripwhite ( line );

		if ( * s ) {
			add_history ( s );
			state = execute_line ( std::string ( s ) );
		}

		free ( line );
	}

	return state;
}

cli::Command::Result Prompt::execute_line ( std::string line ) {
	try {
		cli::Parser parser = cli::Parser ( cli::Lexer ( line ) );
		return parser.parse ( )->run ( m_environment );
	} catch ( const exception::CommonException & exception ) {
		alib::XmlDataFactory::toStdout ( exception );
		return cli::Command::Result::EXCEPTION;
	} catch ( const std::exception & exception ) {
		common::Streams::err << "Exception caught: " << exception.what ( ) << std::endl;
		return cli::Command::Result::EXCEPTION;
	}
}
